<?php
/**
 * Created by PhpStorm.
 * User: francismararaujo
 * Date: 23/10/14
 * Time: 17:07
 */

namespace TecDiet\Services;

use Doctrine\ORM\EntityManager;
use TecDiet\Util\Validation;
use TecDiet\Util\CRUD;
use TecDiet\Services\NovoCodigo;

class EnteralFeedingsService implements Queries {

    protected $entityManager;
    protected $connection;
    protected $crud;
    
    public function __construct(EntityManager $entityManager, CRUD $crud, NovoCodigo $novoCodigo) {

        $this->entityManager = $entityManager;
        $this->connection = $entityManager->getConnection();
        $this->crud = $crud;
        $this->novoCodigo = $novoCodigo;
    }

    private function updateEnteralFeeding($enteralFeeding) {

        $sth = $this->connection->prepare(self::UPDATE_ENTERAL_FEEDING);

        $observation = Validation::removeAccentuation($enteralFeeding['observation'], \PDO::PARAM_STR); 
        $time = Validation::timeToFloat($enteralFeeding['time']);
        $serviceNumbers = $enteralFeeding['serviceNumbers'];
        $plateNumbers = $enteralFeeding['plateNumbers'];
        $serviceNumbersFormat = $serviceNumbers ? Validation::sanitazeNumber($serviceNumbers,false) : "";
        $plateNumbersFormat = $plateNumbers ? Validation::sanitazeNumber($plateNumbers, false) : "";
        
        $sth->bindParam(':CDFILAUXPRAT', $enteralFeeding['plateAuxId']);
        $sth->bindParam(':CDPRATO', $enteralFeeding['formulationId']);
        $sth->bindParam(':DSOBSCONF', $observation);
        $sth->bindParam(':QTSERVCONF', $serviceNumbersFormat);
        $sth->bindParam(':QTPRATCONF', $plateNumbersFormat);
        $sth->bindParam(':QTINFUCONF', $enteralFeeding['qtinfuconf']);
        $sth->bindParam(':HRCONSCONF', $time);
        $sth->bindParam(':DTCONSCONF', $enteralFeeding['enteralDate']);
        $sth->bindParam(':CDPRODUTO', $enteralFeeding['productId']);
        $sth->bindParam(':CDVIACESDIET', $enteralFeeding['accessRoadId']);
        $sth->bindParam(':IDPREPCONF', $enteralFeeding['preparationId']);
        $sth->bindParam(':OLD_CDPACIENTE', $enteralFeeding['id']);
        $sth->bindParam(':OLD_DTSERVINTE', $enteralFeeding['serviceDate']);
        $sth->bindParam(':OLD_NRSEQUSERV', $enteralFeeding['sequential']);
        $sth->bindParam(':OLD_CDSEQUCOPR', $enteralFeeding['unique']);
        $sth->bindParam(':OLD_CDFILIAL', $enteralFeeding['unit']);

        $sth->execute();

    }

    private function getServinteSequential($enteralFeeding) {
        $sth = $this->connection->prepare(self::GET_SEQUENTIAL_SERVINTE);

        $sth->execute(array(
            'PATIENT' => $enteralFeeding['id'],
            'UNIT' => $enteralFeeding['unit'],
            'SERVICE_DATE' => $enteralFeeding['serviceDate'],
            'DTINCLUSAO' => $enteralFeeding['DTINCLUSAO']
        ));
        
        return $sth->fetch(\PDO::FETCH_NUM);

    }

    private function getSequential($enteralFeeding) {
        
        $sth = $this->connection->prepare(self::GET_SEQUENTIAL_CONFESCO);
        $sth->execute(array(
            'PATIENT' => $enteralFeeding['id'],
            'UNIT' => $enteralFeeding['unit'],
            'SERVICE_DATE' => $enteralFeeding['serviceDate']
        ));

        return $sth->fetch(\PDO::FETCH_NUM);
    }

    private function insertEnteralFeeding($enteralFeeding) {
        
        $confescoSequential = $this->novoCodigo->getCodeByTableName('CONFESCO', 1);;
        $servinteSequential = $this->getServinteSequential($enteralFeeding);
        $sth = $this->connection->prepare(self::INSERT_ENTERAL_FEEDING);

        $tmpTime = Validation::timeToFloat($enteralFeeding['time']);
        
        $tmpObservation = explode('.', $enteralFeeding['observation']);
        $enteralFeedingObservation = end($tmpObservation);
        $serviceNumbers = Validation::sanitazeNumber($enteralFeeding['serviceNumbers'],false);
        $plateNumbers = Validation::sanitazeNumber($enteralFeeding['plateNumbers'],false);
        $qtinfuconf = Validation::sanitazeNumber($enteralFeeding['qtinfuconf'],false);
        $sth->bindParam(':CDPACIENTE', $enteralFeeding['id']);
        $sth->bindParam(':DTSERVINTE', $enteralFeeding['serviceDate']);
        $sth->bindParam(':NRSEQUSERV', $servinteSequential[0]);
        $sth->bindParam(':CDSEQUCOPR', $confescoSequential);
        $sth->bindParam(':CDFILAUXPRAT', $enteralFeeding['plateAuxId']);
        $sth->bindParam(':CDPRATO', $enteralFeeding['formulationId']);
        $sth->bindParam(':QTSERVCONF', $serviceNumbers);
        $sth->bindParam(':QTPRATCONF', $plateNumbers);
        $sth->bindParam(':QTINFUCONF', $qtinfuconf);
        $sth->bindParam(':HRCONSCONF', $tmpTime);
        $sth->bindParam(':DTCONSCONF', $enteralFeeding['enteralDate']);
        $sth->bindParam(':CDPRODUTO', $enteralFeeding['productId']);
        $sth->bindParam(':CDVIACESDIET', $enteralFeeding['accessRoadId']);
        $sth->bindParam(':IDPREPCONF', $enteralFeeding['preparationId']);
        $sth->bindParam(':DSOBSCONF', $enteralFeedingObservation);
        $sth->bindParam(':CDFILIAL', $enteralFeeding['unit']);

        $sth->execute();
        return $confescoSequential;
    }

    public function save($enteralFeedings) {
        $erroToSaveEnteralFeeding = array(
            'ENTERALFEEDING' => array(),
            'ERROR' => array(),
            'OPERATION' => array(),
            'SEQUENTIAL' => array()
        );
        foreach ($enteralFeedings as $enteralFeeding) {
            if($enteralFeeding['__is_new'] == true) {
                try{
                    $sequential = $this->insertEnteralFeeding($enteralFeeding);
                    array_push($erroToSaveEnteralFeeding['SEQUENTIAL'], $sequential);
                }catch(\Exception $e){
                    array_push($erroToSaveEnteralFeeding['ENTERALFEEDING'], $enteralFeeding);
                    array_push($erroToSaveEnteralFeeding['ERROR'], $e->getMessage().' Error Code: '.$e->getCode());
                    array_push($erroToSaveEnteralFeeding['OPERATION'], 'INSERT');
                    array_push($erroToSaveEnteralFeeding['SEQUENTIAL'], null);
                }
            } else {
                try {
                    $this->updateEnteralFeeding($enteralFeeding);
                } catch (\Exception $e) {
                    array_push($erroToSaveEnteralFeeding['ENTERALFEEDING'], $enteralFeeding);
                    array_push($erroToSaveEnteralFeeding['ERROR'], $e->getMessage().' Error Code: '.$e->getCode());
                    array_push($erroToSaveEnteralFeeding['OPERATION'], 'UPDATE');
                }
            }
        }
        return $erroToSaveEnteralFeeding;
    }
    
    public function delete($enteralFeedings){
        $tableName = 'CONFESCO';
        $erroToDeleteEnteralFeeding = array(
            'ENTERALFEEDING' => array(),
            'ERROR' => array(),
            'OPERATION' => 'DELETE'
        );
        foreach($enteralFeedings as $enteralFeeding){
           if(isset($enteralFeeding['unique'])){
               try {
                   $enteralFeeding["cdfilial"] = $enteralFeeding['unit'];
                   $enteralFeeding["cdsequcopr"] = $enteralFeeding['unique'];
                   $enteralFeeding["nrsequserv"] = $enteralFeeding['sequential'];
                   $enteralFeeding["dtservinte"] = $enteralFeeding['serviceDate'];
                   $enteralFeeding["cdpaciente"] = $enteralFeeding['id'];
                   if(isset($enteralFeeding['inconsistenceToDelete']) && $enteralFeeding['inconsistenceToDelete'] == true){
                        throw new \Exception('Delete propagado, porém houve erro ao remover registro pai');
                    }
                    $this->crud->delete($tableName, $enteralFeeding);
               } catch (\Exception $e) {
                    array_push($erroToDeleteEnteralFeeding['ENTERALFEEDING'], $enteralFeeding);
                    array_push($erroToDeleteEnteralFeeding['ERROR'], $e->getMessage().' Error Code: '.$e->getCode());
               }
           }
        }
    }

}