<?php
/**
 * Created by PhpStorm.
 * User: francismararaujo
 * Date: 23/10/14
 * Time: 17:07
 */

namespace TecDiet\Services;

use Doctrine\ORM\EntityManager;
use TecDiet\Util\Validation;
use TecDiet\Util\CRUD;
use TecDiet\Services\NovoCodigo;

class LactaryFeedingsService implements Queries {

    protected $entityManager;
    protected $connection;
    protected $sequential;
    protected $crud;
    protected $novoCodigo;
    
    public function __construct(EntityManager $entityManager, CRUD $crud, NovoCodigo $novoCodigo) {

        $this->entityManager = $entityManager;
        $this->connection = $entityManager->getConnection();
        $this->crud = $crud;
        $this->novoCodigo = $novoCodigo;
    }
    
    private function getSequential($lactaryFeeding) {
        $sth = $this->connection->prepare(self::GET_SEQUENTIAL_SERVINTE);
        $sth->execute(array(
            'PATIENT' => $lactaryFeeding['id'],
            'UNIT' => $lactaryFeeding['unit'],
            'SERVICE_DATE' => $lactaryFeeding['serviceDate'],
            'DTINCLUSAO' => $lactaryFeeding['DTINCLUSAO']
        ));
        
        return $sth->fetch(\PDO::FETCH_NUM);

    }

    private function getSequentialConfesco($lactaryFeeding) {
        $sth = $this->connection->prepare(self::GET_SEQUENTIAL_CONFESCO);
        $sth->execute(array(
            'PATIENT' => $lactaryFeeding['id'],
            'UNIT' => $lactaryFeeding['unit'],
            'SERVICE_DATE' => $lactaryFeeding['serviceDate']
        ));
        return current($sth->fetch(\PDO::FETCH_NUM));
    }

    private function updateLactaryFeeding($lactaryFeeding) {

        $sth = $this->connection->prepare(self::UPDATE_LACTARY_FEEDING);
        
        $sequentialServinte = self::getSequential($lactaryFeeding);
        
        $observationWithoutAccents = Validation::removeAccentuation($lactaryFeeding['observation']);
        $convertedTimeToFloat = Validation::timeToFloat($lactaryFeeding['hourLactaryPlate']);
        $plateNumbers = Validation::sanitazeNumber($lactaryFeeding['plateNumbers'], false);
        $sth->bindParam(':DSOBSCONF', $observationWithoutAccents);
        $sth->bindParam(':CDFILAUXPRAT', $lactaryFeeding['plateAuxId']);
        $sth->bindParam(':CDPRATO', $lactaryFeeding['formulationId']);
        $sth->bindParam(':QTPRATCONF', $plateNumbers);
        $sth->bindParam(':HRCONSCONF', $convertedTimeToFloat);
        $sth->bindParam(':DTCONSCONF', $lactaryFeeding['lactaryDate']);
        $sth->bindParam(':OLD_CDPACIENTE', $lactaryFeeding['id']);
        $sth->bindParam(':OLD_DTSERVINTE', $lactaryFeeding['serviceDate']);
        $sth->bindParam(':OLD_NRSEQUSERV', $sequentialServinte[0]);
        $sth->bindParam(':OLD_CDSEQUCOPR', $lactaryFeeding['unique']);
        $sth->bindParam(':OLD_CDFILIAL', $lactaryFeeding['unit']);

        $sth->execute();

    }

    private function insertLactaryFeeding($lactaryFeeding) {
        
        // $sequentialConfesco = self::getSequentialConfesco($lactaryFeeding);
        $sequentialConfesco = $this->novoCodigo->getCodeByTableName('CONFESCO', 1);
        $sequentialServinte = self::getSequential($lactaryFeeding);
        $sth = $this->connection->prepare(self::INSERT_LACTARY_FEEDING);
    
        if(!isset($lactaryFeeding['observation'])){
            $lactaryFeeding['observation'] = "";
        }
        $sth->execute(array(
            'CDPACIENTE' => $lactaryFeeding['id'],
            'DTSERVINTE' => $lactaryFeeding['serviceDate'],
            'NRSEQUSERV' => $sequentialServinte[0],
            'CDSEQUCOPR' => $sequentialConfesco,
            'DSOBSCONF' => Validation::removeAccentuation($lactaryFeeding['observation'], \PDO::PARAM_STR),
            'CDFILAUXPRAT' => $lactaryFeeding['plateAuxId'],
            'CDPRATO' => $lactaryFeeding['formulationId'],
            'QTPRATCONF' => Validation::sanitazeNumber($lactaryFeeding['plateNumbers'], false),
            'HRCONSCONF' => Validation::timeToFloat($lactaryFeeding['hourLactaryPlate']),
            'DTCONSCONF' => $lactaryFeeding["lactaryDate"],
            'CDFILIAL' => $lactaryFeeding['unit']
        ));
        return $sequentialConfesco;
    }

    public function save($lactaryFeedings) {
        $erroToSaveLactaryFeeding = array(
            'LACTARYFEEDING' => array(),
            'ERROR' => array(),
            'OPERATION' => array(),
            'SEQUENTIAL' => array()
        );
        foreach ($lactaryFeedings as $lactaryFeeding) {
            if($lactaryFeeding['__is_new'] == true) {
                 try {
                    $sequential = $this->insertLactaryFeeding($lactaryFeeding);
                    array_push($erroToSaveLactaryFeeding['SEQUENTIAL'], $sequential);
                 } catch (\Exception $e) {
                     array_push($erroToSaveLactaryFeeding['LACTARYFEEDING'], $lactaryFeeding);
                     array_push($erroToSaveLactaryFeeding['ERROR'], $e->getMessage().' Error Code: '.$e->getCode());
                     array_push($erroToSaveLactaryFeeding['OPERATION'], 'INSERT');
                     array_push($erroToSaveLactaryFeeding['SEQUENTIAL'], null);
                 }
            } else {
                try {
                    $this->updateLactaryFeeding($lactaryFeeding);
                } catch (\Exception $e) {
                    array_push($erroToSaveLactaryFeeding['LACTARYFEEDING'], $lactaryFeeding);
                    array_push($erroToSaveLactaryFeeding['ERROR'], $e->getMessage().' Error Code: '.$e->getCode());
                    array_push($erroToSaveLactaryFeeding['OPERATION'], 'UPDATE');
                }
            }
        }
        return $erroToSaveLactaryFeeding;
    }
    
    public function delete($lactaryFeedings){ 
        $tableName = 'CONFESCO';
        $erroToDeleteLactaryFeeding = array(
            'LACTARYFEEDING' => array(),
            'ERROR' => array(),
            'OPERATION' => 'DELETE'
        );
        foreach ($lactaryFeedings as $lactaryFeeding){
            if(isset($lactaryFeeding['unique'])){
                try {
                    if(isset($lactaryFeeding['inconsistenceToDelete']) && $lactaryFeeding['inconsistenceToDelete'] == true){
                        throw new \Exception('Delete propagado, porém houve erro ao remover registro pai');
                    }
                    $lactaryFeeding["cdfilial"] = $lactaryFeeding["unit"];
                    $lactaryFeeding["cdsequcopr"] = $lactaryFeeding["unique"];
                    $lactaryFeeding["nrsequserv"] = $lactaryFeeding["sequential"];
                    $lactaryFeeding["dtservinte"] = $lactaryFeeding["serviceDate"];
                    $lactaryFeeding["cdpaciente"] = $lactaryFeeding["id"];
                    $this->crud->delete($tableName, $lactaryFeeding);
                } catch (\Exception $e) {
                    array_push($erroToDeleteLactaryFeeding['LACTARYFEEDING'], $lactaryFeeding);
                    array_push($erroToDeleteLactaryFeeding['ERROR'], $e->getMessage().' Error Code: '.$e->getCode());
                }
            }
        }
        return $erroToDeleteLactaryFeeding;
    }

}