<?php
/**
 * Created by PhpStorm.
 * User: francismararaujo
 * Date: 23/10/14
 * Time: 17:07
 */

namespace TecDiet\Services;

use Doctrine\ORM\EntityManager;
use BusinessCommon\Services\Environment;
use Util\Validation;

class MapNutritionistService implements Queries {

    protected $entityManager;
    protected $environment;
    protected $connection;
    protected $patients = array();
    protected $storeys = array();
    protected $halls = array();
    protected $hallsDistincts = array();
    protected $beds = array();

    public function __construct(EntityManager $entityManager, Environment $environment) {

        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->connection = $entityManager->getConnection();

    }

    public function save($patients) {

        $patientRepostiory = $this->entityManager->getRepository('Model\Paciente');

        foreach ($patients as $patient) {

            $updateInternac = $this->connection->prepare(self::UPDATE_INTERNAC);

            $updateInternac->execute(
                array( 
                    'SITUATION' => $patient['situation'],
                    'DISCHARGEDATE' => $patient['dischargeDate'],
                    'PATIENT' => $patient['id'],
                    'UNITS' => $patient['unit']
                )
            );

            $updateLeito = $this->connection->prepare(self::UPDATE_LEITO);

            $updateLeito->execute(
                array(
                    'PATIENT' => $patient['id'],
                    'UNITS' => $patient['unit']
                )
            );

        }

    }

    private function getAnthMethods($patient, $appointmentData) {

        $age = Validation::getAge($patient['DTNASCPACI']);

        $sth = $this->connection->prepare(self::ANTHROPOMETRIC_EVALUATION);

        $sth->execute(array(
            'AGE' => filter_var($age, FILTER_SANITIZE_NUMBER_INT),
            'APPOINTMENTDATE' => $appointmentData,
            'UNITS' => $patient['CDFILIAL'],
            'PATIENT' => $patient['CDPACIENTE']
        ));

        return $sth->fetchAll(\PDO::FETCH_ASSOC);

    }

    private function getAppointments($patient) {

        $sth = $this->connection->prepare(self::APPOINTMENTS);

        $sth->execute(array(
            'PATIENT' => $patient['CDPACIENTE'],
            'UNITS' => $patient['CDFILIAL']
        ));

        return $sth->fetchAll(\PDO::FETCH_ASSOC);

    }

    private function getPrescriptions($patient) {

        $sth = $this->connection->prepare(self::PRESCRIPTIONS);
        
        $sth->execute(array(
            'PATIENT' => $patient['CDPACIENTE'],
            'UNITS' => $patient['CDFILIAL']
        ));

        return $sth->fetchAll(\PDO::FETCH_ASSOC);

    }

    private function getProfile($patient) {

        $sth = $this->connection->prepare(self::PROFILES);

        $sth->execute(array(
            'PATIENT' => $patient['CDPACIENTE'],
            'UNITS' => $patient['CDFILIAL']
        ));

        return $sth->fetch(\PDO::FETCH_ASSOC);

    }

    private function initBeds($unit) {

        $sth = $this->connection->prepare(self::BEDS);

        $sth->execute(array(
            'UNITS' => $unit
        ));

        $beds = $sth->fetchAll(\PDO::FETCH_ASSOC);

        foreach ($beds as $bed) {

            $this->beds[] = array(
                'storey' => $bed['CDANDAR'],
                'hall' => $bed['CDALA'],
                'bed' => $bed['CDLEITO']
            );

        }

    }

    private function initHalls($unit) {

        $sth = $this->connection->prepare(self::HALLS);

        $sth->execute(array(
            'UNITS' => $unit
        ));

        $hallsDistincts = $sth->fetchAll(\PDO::FETCH_ASSOC);

        foreach ($hallsDistincts as $hall) {

            $this->hallsDistincts[] = array(
                'storey' => $hall['CDANDAR'],
                'hall' => $hall['CDALA']
            );

        }

    }
    private function initHallsDistincts($unit) {

        $sth = $this->connection->prepare(self::HALLS_DISTINCT);

        $sth->execute(array(
            'UNITS' => $unit
        ));

        $halls = $sth->fetchAll(\PDO::FETCH_ASSOC);

        foreach ($halls as $hall) {
            $this->halls[] = array(
                'hall' => $hall['CDALA']
            );
        }
    }

    private function initStoreys($unit) {

        $sth = $this->connection->prepare(self::STOREYS);

        $sth->execute(array(
            'UNITS' => $unit
        ));

        $storeys = $sth->fetchAll(\PDO::FETCH_ASSOC);

        foreach ($storeys as $storey) {

            $this->storeys[] = array(
                'storey' => $storey['CDANDAR']
            );

        }

    }

    private function initPatients($unit) {

        $sth = $this->connection->prepare(self::PATIENTS);

        $sth->execute(array(
            'UNITS' => $unit
        ));

        $patients = $sth->fetchAll(\PDO::FETCH_ASSOC);

        foreach ($patients as $patient) {

            $this->patients['main'][] = array(

                'id' => $patient['CDPACIENTE'],
                'age' => Validation::getAge($patient['DTNASCPACI']),
                'name' => $patient['NMPACIUPPER'],
                'situation' => $patient['IDMOTIALTA'],
                'storey' => $patient['CDANDAR'],
                'hall' => $patient['CDALA'],
                'bed' => $patient['CDLEITO'],
                'dischargeDate' => $patient['DTALTA'],
                'unit' => $patient['CDFILIAL']

            );

            $profile = $this->getProfile($patient);
            
            $this->patients['profiles'][] = array(
                'id' => $patient['CDPACIENTE'],
                'confidentiality' => $profile['DSOBSCONFID'],
                'observation' => $profile['DSOBSGERAL'],
                'eatingHabits' => Validation::toPlainText($profile['TXHABIALIM']),
                'foodIntolerance' => Validation::toPlainText($profile['TXINTOALIM'])
            );

            $prescriptions = $this->getPrescriptions($patient);

            foreach ($prescriptions as $prescription) {

                $this->patients['prescriptions'][] = array(
                    'id' => $patient['CDPACIENTE'],
                    'sequential' => $prescription['ROWNUM'],
                    'date' => $prescription['DTPRESDIET'],
                    'bed' => $prescription['CDLEITO'],
                    'diet' => $prescription['DSDIETPAD'],
                    'prescription' => $prescription['DSPRESDIET'],
                    'bedName'   => $prescription['NMLEITO']
                );

            }

            $appointments = $this->getAppointments($patient);
            
            foreach ($appointments as $appointment) {

                $this->patients['appointments'][] = array(
                    'id' => $patient['CDPACIENTE'],
                    'sequential' => $appointment['ROWNUM'],
                    'date' => $appointment['DTCONSPACI'],
                    'height' => floatval($appointment['ALTURA']),
                    'weight' => floatval($appointment['PESO']),
                    'diet' => $appointment['DSDIETPAD'],
                    'healthInsurance' => $appointment['NMSERPADDIE'],
                    'nutritionalStatus' => $appointment['NMNIASNUTR'],
                    'medicalTeam' => $appointment['NMEQUIMED'],
                    'nutritionist' => $appointment['NMOPERADOR']
                );

                // ANTHROPOMETRIC EVALUATION
                $anthMethods = $this->getAnthMethods($patient, $appointment['DTCONSPACI']);

                foreach($anthMethods as $anthMethod) {

                    $this->patients['anthEvaluations']['methods'][] = array(
                        'id' => $anthMethod['CDPACIENTE'],
                        'name' => $anthMethod['NMTBMETOAVAL'],
                        'method' => $anthMethod['CDTBMETOAVAL'],
                        'date' => $anthMethod['DTCONSPACI'],
                        '__isSelected' => Validation::isSelected($anthMethod['IDTABELAAV'])
                    );

                }

            }

        }

    }

    public function synchronism() {
        
        $units = $this->environment->getUnitsId();
        
        foreach ($units as $unit) {

            $this->initPatients($unit);
            $this->initStoreys($unit);
            $this->initHalls($unit);
            $this->initHallsDistincts($unit);
            $this->initBeds($unit);

        }

    }

    public function getDataSet() {

        return array(
            'patients' => $this->patients['main'],
            'profiles' => $this->patients['profiles'],
            'prescriptions' => $this->patients['prescriptions'],
            'appointments' => $this->patients['appointments'],
            'anthEvaluationMethods' => $this->patients['anthEvaluations']['methods'],
            'storeys' => $this->storeys,
            'halls' => $this->halls,
            'hallsDistincts' => $this->hallsDistincts,
            'beds' => $this->beds
        );

    }

}