describe("PackageMetadata", function() {
    var _restEngine;
    var $httpBackend;
    var $rootScope;
    var jSONContainerLoader;
    Configuration(function(restEngine) {
        _restEngine = restEngine;
    });

    var buildMock = function() {
        var mock = {
            whenSuccess: function() {},
            whenError: function() {}
        };
        spyOn(mock, 'whenSuccess').and.callThrough();
        spyOn(mock, 'whenError').and.callThrough();
        return mock;
    };

    beforeEach(function() {
        window.templateUrl = "../../frontend/templates/dyman/";
        window.metadataUrl = "./";
        window.metadataUrls = [{
            "package": "identity",
            "baseUrl": "http://identity.zeedhi.com/frontend/",
        }, {
            "package": "pm",
            "baseUrl": "http://pm.zeedhi.com/frontend/",
        }, {
            "package": "devOps",
            "baseUrl": "http://devops.zeedhi.com/frontend/",
        }];
        window.serviceUrl = "../backend/service/index.php";
        window.serviceUrls = [{
            "package": "identity",
            "baseUrl": "http://identity.zeedhi.com/backend/service/index.php",
        }, {
            "package": "pm",
            "baseUrl": "http://pm.zeedhi.com/backend/service/index.php"
        }, {
            "package": "devOps",
            "baseUrl": "http://devops.zeedhi.com/backend"
        }];
        console.error = function() {};
        module('ZeedhiApp');
    });

    beforeEach(inject(function(_$rootScope_, _$httpBackend_, JSONContainerLoader) {
        // _$httpBackend_.when("POST", '../backend/service/index.php/success/find').respond({"data":[]}, {});
        // _$httpBackend_.when("POST", '../backend/service/index.php/error/find').respond({"error":"deu erro"}, {});

        // _$httpBackend_.when("GET", '/json_test/test/success/find').respond({ "data":{} }, {});
        // _$httpBackend_.when("GET", '/json_test/test/error/find').respond(500, {"error":{message: "Unexpected token"}}, {});
        // _$httpBackend_.when("GET", '/json_test2/test/error/find').respond(500, {"error":{message: "Unexpected token"}}, {});
        $rootScope = _$rootScope_;
        $httpBackend = _$httpBackend_;
        jSONContainerLoader = JSONContainerLoader;
    }));

    it("The requestMetaData method should return a valid json inside the default package", function(done) {
        $httpBackend.expectGET("./json/containers/login.json").respond({
            "data": {}
        });

        var mock = buildMock();
        _restEngine.requestMetaData("json/containers/login.json", {})
            .then(mock.whenSuccess)
            .catch(mock.whenError)
            .then(function() {
                expect(mock.whenSuccess).toHaveBeenCalled();
                expect(mock.whenError).not.toHaveBeenCalled();
                done();
            });
        $httpBackend.flush();
    });

    it("The requestMetaData method should return a valid json inside some folder in the default package", function(done) {
        $httpBackend.expectGET("./json/containers/folderA/login.json").respond(200, {});

        var mock = buildMock();
        _restEngine.requestMetaData("json/containers/folderA/login.json", {})
            .then(mock.whenSuccess)
            .catch(function(e) {
                console.log(e);
                mock.whenError();
            })
            .then(function() {
                expect(mock.whenSuccess).toHaveBeenCalled();
                expect(mock.whenError).not.toHaveBeenCalled();
                done();
            });
        $httpBackend.flush();
    });

    it("The requestMetaData method should return a valid json inside some provided package", function(done) {
        $httpBackend.expectGET("http://identity.zeedhi.com/frontend/json/containers/login.json").respond(200, {});

        var mock = buildMock();
        _restEngine.requestMetaData("identity#json/containers/login.json", {})
            .then(mock.whenSuccess)
            .catch(mock.whenError)
            .then(function() {
                expect(mock.whenSuccess).toHaveBeenCalled();
                expect(mock.whenError).not.toHaveBeenCalled();
                done();
            });
        $httpBackend.flush();
    });

    it("The requestMetaData method should return a valid json inside some folder in some provided package", function(done) {
        $httpBackend.expectGET("http://devops.zeedhi.com/frontend/json/containers/folderO/login.json").respond(200, {});

        var mock = buildMock();
        _restEngine.requestMetaData("devOps#json/containers/folderO/login.json", {})
            .then(mock.whenSuccess)
            .catch(mock.whenError)
            .then(function() {
                expect(mock.whenSuccess).toHaveBeenCalled();
                expect(mock.whenError).not.toHaveBeenCalled();
                done();
            });
        $httpBackend.flush();
    });

    it("The get request method should return a valid json inside some folder in some provided package", function(done) {
        $httpBackend.expectGET("http://devops.zeedhi.com/backend/save").respond(200, {});
        var mock = buildMock();
        _restEngine.get("devOps#/save")
            .then(mock.whenSuccess)
            .catch(mock.whenError)
            .then(function() {
                expect(mock.whenSuccess).toHaveBeenCalled();
                expect(mock.whenError).not.toHaveBeenCalled();
                done();
            });
        $httpBackend.flush();
    });

    it("The post request method should return a valid json inside some folder in some provided package", function(done) {
        $httpBackend.expectPOST("http://devops.zeedhi.com/backend/save").respond(200, {});
        var mock = buildMock();
        _restEngine.post("devOps#/save")
            .then(mock.whenSuccess)
            .catch(mock.whenError)
            .then(function() {
                expect(mock.whenSuccess).toHaveBeenCalled();
                expect(mock.whenError).not.toHaveBeenCalled();
                done();
            });
        $httpBackend.flush();
    });
    it("The delete request method should return a valid json inside some folder in some provided package", function(done) {
        $httpBackend.expectDELETE("http://devops.zeedhi.com/backend/delete").respond(200, {});
        var mock = buildMock();
        _restEngine.delete("devOps#/delete")
            .then(mock.whenSuccess)
            .catch(mock.whenError)
            .then(function() {
                expect(mock.whenSuccess).toHaveBeenCalled();
                expect(mock.whenError).not.toHaveBeenCalled();
                done();
            });
        $httpBackend.flush();
    });
    it("The put request method should return a valid json inside some folder in some provided package", function(done) {
        $httpBackend.expectPUT("http://devops.zeedhi.com/backend/save").respond(200, {});
        var mock = buildMock();
        _restEngine.put("devOps#/save")
            .then(mock.whenSuccess)
            .catch(mock.whenError)
            .then(function() {
                expect(mock.whenSuccess).toHaveBeenCalled();
                expect(mock.whenError).not.toHaveBeenCalled();
                done();
            });
        $httpBackend.flush();
    });

    describe("Integration tests", function() {
        it("The jSONContainerLoader should request the rigth url from the conainer", function(done) {
            //
            $httpBackend.expectGET("http://devops.zeedhi.com/frontend/json/containers/folderO/login.json").respond(200, {});

            var mock = buildMock();
            jSONContainerLoader.loadContainer("devOps#folderO/login")
                .then(mock.whenSuccess)
                .catch(mock.whenError)
                .then(function() {
                    expect(mock.whenSuccess).toHaveBeenCalled();
                    expect(mock.whenError).not.toHaveBeenCalled();
                    done();
                });
            $httpBackend.flush();
        });
    });




});