describe("ZhLocation", function() {

	// mock navigator
	navigator.__defineGetter__('geolocation', function() {
		return {
			isProviderEnabled: function() {
				return true;
			},
			isAvailableProvider: this.isProviderEnabled,
			getCurrentPosition: function(success, error, options) {
				setTimeout(function() {
					success({
						coords: {
							accuracy: 59,
							altitude: null,
							altitudeAccuracy: null,
							heading: null,
							latitude: -19.9326553,
							longitude: -43.9306224,
							speed: null
						},
						timestamp: new Date().getTime()
					});
				}, 3000);
			},
			watchPosition: function(success, error, options) {
				this.getCurrentPosition(success, error, options);
				return new Date().getTime().toString();
			},
			clearWatch: function(id) {
				return true;
			}
		};
	});

	var screenService = {
		i18n: function(message) {
			return message;
		}
	};
	var zhPromise = new ZHPromise(Q);
	var zhLocation = new ZhLocation(zhPromise, screenService);

	// all tests below use browser location
	it("Returns the current position", function(done) {
		var processed = false;
		zhLocation.getCurrentPosition().then(function(position) {
			expect(position.coords.accuracy).toEqual(59);
			expect(position.coords.latitude).toEqual(-19.9326553);
			expect(position.coords.longitude).toEqual(-43.9306224);
			processed = true;
			done();
		});
	});

	it("Watch position", function(done) {
		var processed = false;
		var positionCallback = {
			changePosition: function() {},
			errorCallback: function() {}
		};
		spyOn(positionCallback, 'changePosition').and.callFake(function() {
			expect(positionCallback.changePosition).toHaveBeenCalled();
			done();
		});

		zhLocation.watchPosition(null, positionCallback.changePosition, positionCallback.errorCallback);
	});
});