describe("ZhUtil", function() {

	var zhUtil;
	var dataSourceHolder;
	var zhPromise = new ZHPromise(Q);
	dataSourceHolder = new DataSourceHolder(zhPromise, {}, {});
	var baseContainer = ZhHelper.baseContainerWithTwoWidgets;
	var baseDataSourceData = ZhHelper.getTestDataSourceDataWithNames;
	var baseFields = ZhHelper.baseFields;
	var widget;

	beforeEach(function () {
		module("ZeedhiApp");
		module("ZeedhiServices");

		inject(function($injector, metaDataFactory){
			var container = metaDataFactory.containerFactory(baseContainer());
			widget = container.widgets[0];
			$filter = $injector.get('$filter');
			zhUtil = new ZhUtil({ getArgsToFunction: function() { return { keys: ['functions'], args: [window.functions] }; } }, new QueryBuilder(), $filter, dataSourceHolder);
		});
	});

	window.functions = {
		"string": function(){ return 'teste'; },
		"number": function(){ return 23; },
		"array": function(){ return [1,2,3]; },
		"undefined": function(){ return undefined; },
		"null": function(){ return null; },
		"boolF": function(){ return false; },
		"boolT": function(){ return true; },
		"object": function(){ return {a: 'aa'}; }
	};

	var _simpleValueAsserts = function(util) {
		expect(util.evaluate('teste')).toEqual('teste');
		expect(util.evaluate(123)).toEqual(123);
		expect(util.evaluate([1,2,3])).toEqual([1,2,3]);
		expect(util.evaluate(false)).toEqual(false);
		expect(util.evaluate(true)).toEqual(true);
		expect(util.evaluate(null)).toEqual(null);
		expect(util.evaluate(undefined)).toEqual(undefined);
		expect(util.evaluate({a: 'a'})).toEqual({a: 'a'});
	};

	var _functionsValueAsserts = function(util) {
		expect(util.evaluate(functions.string)).toEqual('teste');
		expect(util.evaluate(functions.number)).toEqual(23);
		expect(util.evaluate(functions.array)).toEqual([1,2,3]);
		expect(util.evaluate(functions.undefined)).toEqual(undefined);
		expect(util.evaluate(functions.null)).toEqual(null);
		expect(util.evaluate(functions.boolF)).toEqual(false);
		expect(util.evaluate(functions.boolT)).toEqual(true);
		expect(util.evaluate(functions.object)).toEqual({a: 'aa'});
	};

	var _evaluatedFunctionsAsserts = function(util){
		expect(util.evaluate('@functions.string()@')).toEqual('teste');
		expect(util.evaluate('@functions.number()@')).toEqual(23);
		expect(util.evaluate('@functions.number() + functions.number()@')).toEqual(46);
		expect(util.evaluate('@functions.array()@')).toEqual([1,2,3]);
		expect(util.evaluate('@functions.undefined()@')).toEqual(undefined);
		expect(util.evaluate('@functions.null()@')).toEqual(null);
		expect(util.evaluate('@functions.boolF()@')).toEqual(false);
		expect(util.evaluate('@functions.boolT()@')).toEqual(true);
		expect(util.evaluate('@functions.object()@')).toEqual({a: 'aa'});
	};

	var _evaluatedFunctionsWithPrefixAsserts = function(util){
		expect(util.evaluate('My value is: @functions.string()@')).toEqual('My value is: teste');
		expect(util.evaluate('My value is: @functions.number()@')).toEqual('My value is: 23');
		expect(util.evaluate('My value is: @functions.number() + functions.number()@')).toEqual('My value is: 46');
		expect(util.evaluate('My value is: @functions.array()@')).toEqual('My value is: 1,2,3');
		expect(util.evaluate('My value is: @functions.undefined()@')).toEqual('My value is: undefined');
		expect(util.evaluate('My value is: @functions.null()@')).toEqual('My value is: null');
		expect(util.evaluate('My value is: @functions.boolF()@')).toEqual('My value is: false');
		expect(util.evaluate('My value is: @functions.boolT()@')).toEqual('My value is: true');
		expect(util.evaluate('My value is: @functions.object()@')).toEqual('My value is: [object Object]');

		expect(util.evaluate('@functions.number()@_@functions.string()@')).toEqual('23_teste');
		expect(util.evaluate('@functions.number()@_@functions.number()@')).toEqual('23_23');
		expect(util.evaluate('@functions.number()@_@functions.array()@')).toEqual('23_1,2,3');
		expect(util.evaluate('@functions.number()@_@functions.undefined()@')).toEqual('23_undefined');
		expect(util.evaluate('@functions.number()@_@functions.null()@')).toEqual('23_null');
		expect(util.evaluate('@functions.number()@_@functions.boolF()@')).toEqual('23_false');
		expect(util.evaluate('@functions.number()@_@functions.boolT()@')).toEqual('23_true');
		expect(util.evaluate('@functions.number()@_@functions.object()@')).toEqual('23_[object Object]');
	};

	it('ZhUtil.evaluate: if the value is simple, always must be return a origin value', function() {
		_simpleValueAsserts(zhUtil);
	});

	it('ZhUtil.evaluate: if the value is a function, always must be return a function return value', function() {
		_functionsValueAsserts(zhUtil);
	});

	it('ZhUtil.evaluate: if the value is a string with one function, always must be return a function return value', function() {
		_evaluatedFunctionsAsserts(zhUtil);
	});

	it('ZhUtil.evaluate: if the value is a string with a function and prefix, or other function, always must be return a function return value in string format', function() {
		_evaluatedFunctionsWithPrefixAsserts(zhUtil);
	});

	it('ZhUtil.formatLangToMobiscroll: format language to the format accepted by mobiscroll', function() {
		expect(zhUtil.formatLangToMobiscroll('pt_br')).toEqual('pt-BR');
	});

	it('ZhUtil.clearRow: should clone the row and remove some attributes', function(){
		var row = {
			propA: 'aaaaa',
			propB: 'bbbbb',
			__rightActions: [{}, {}],
			__leftActions: [{}, {}],
			'$error': '$error',
			__proxy_: "uabafet"
		};

		var newRow = zhUtil.clearRow(row);

		expect(newRow.propA).toEqual('aaaaa');
		expect(newRow.propB).toEqual('bbbbb');
		expect(newRow.__rightActions).toEqual(undefined);
		expect(newRow.__leftActions).toEqual(undefined);
		expect(newRow.$error).toEqual(undefined);
		expect(newRow.__proxy).toEqual(undefined);
	});

	it('ZhUtil.clearRow: should just return the row if none is passed', function () {
		expect(zhUtil.clearRow(null)).toBe(null);
	});

	it('ZhUtil.getLang: should return the window lang or en_us', function () {
		var previousWindowLang = window.lang;
		window.lang = 'pt_br';

		expect(zhUtil.getLang()).toEqual(window.lang);

		window.lang = null;
		expect(zhUtil.getLang()).toEqual('en_us');

		window.lang = previousWindowLang;
	});

	it('ZhUtil.registerDataSource: should register the dataSource', function(done) {
		var dataSourceName = 'dsuaba';
		var localKey = ['uaba'];

		zhUtil.registerDataSource(dataSourceName, localKey);

		dataSourceHolder.getDataSource(dataSourceName)
			.then(function (newDataSource) {
				expect(newDataSource.dsuaba.localKey).toEqual(localKey);
				done();
			});

	});

	it('ZhUtil.getInColumns: should return inColumns', function () {
		var widget = {
			currentRow: {
				uaba: "fet"
			}
		};

		var field = {
			inData: {
				fet: "uaba"
			}
		};

		var inColumns = zhUtil.getInColumns(field, widget);

		expect(inColumns.fet).toEqual("fet");
	});

	it('ZhUtil.buildFiltersInField : should return a correct filter', function () {
		var widget = {
			currentRow: {
				uaba: "fet"
			}
		};

		var field = {
			inData: {
				fet: "uaba"
			},
			dataSourceFilter: [
				{
					"name": "NAME",
					"value": "a",
					"operator": "LIKE"
				}
			],
			widget: widget
		};

		var filter = zhUtil.buildFiltersInField(field, 'a');

		expect(filter.length).toBe(2);
		expect(filter[0].name).toEqual('NAME');
		expect(filter[0].operator).toEqual('LIKE');
		expect(filter[0].value).toEqual('a');
		expect(filter[1].name).toEqual('fet');
		expect(filter[1].operator).toEqual('=');
		expect(filter[1].value).toEqual('fet');
	});

	it('ZhUtil.orderBy: should order an array of objects by a specified property', function(){
		var values = [
			{id: 1, name: "Zend"},
			{id: 2, name: "AngularJS"}
		];
		var orderedById = values;
		var orderedByName = [
			{id: 2, name: "AngularJS"},
			{id: 1, name: "Zend"}
		];

		expect(zhUtil.orderBy(values, 'id')).toEqual(orderedById);
		expect(zhUtil.orderBy(values, 'name')).toEqual(orderedByName);
	});

	it("getLikeAllFilter: should return an empty array if the searchCriteria is empty or blank", function() {
		widget.dataSource.data = baseDataSourceData();
		widget.fields = baseFields();

		expect(zhUtil.getLikeAllFilter(widget, {})).toEqual([]);
	});

	it("getLikeAllFilter: should return a 'like all' filter of the ACL visible fields", function() {
		widget.dataSource.data = baseDataSourceData();
		widget.fields = baseFields();

		expect(zhUtil.getLikeAllFilter(widget, { "_ALL": "Rodrigo" })).toEqual([{
			name: "name|birth|genre|last|hu",
			operator: Query.LIKE_ALL,
			value: "%Rodrigo%"
		}]);

		widget.fields[0].aclIsVisible = false;
		expect(zhUtil.getLikeAllFilter(widget, { "_ALL": "Rodrigo" })).toEqual([{
			name: "birth|genre|last|hu",
			operator: Query.LIKE_ALL,
			value: "%Rodrigo%"
		}]);
	});

});