describe("I18N", function() {

	var ZHPromise = Q;

	var restEngine = {
		requestMetaData: function() {}
	};

	var eventAggregator = {
		publish: function() {}
	};

	var localVars = {};
	var localStorageEngine = {
		getLocalVar: function(name) {
			return localVars[name];
		},
		setLocalVar: function(name, value) {
			localVars[name] = value;
		}
	};

	var _$injector = {
		get: function (service) {
			switch (service) {
				case 'ZHPromise':			return ZHPromise;
				case 'restEngine':			return restEngine;
				case 'eventAggregator':		return eventAggregator;
				case 'localStorageEngine':	return localStorageEngine;
			}
		}
	};

	var words = {
		"pt_br": {
			"name": "nome",
			"key": "__myVar__ are important"
		},
		"en_us": {
			"name": "name",
			"key": "replacing with __myVar__"
		},
		"es": {
			"name": "nombre",
			"key": "las __myVar__ son importantes"
		},
		"default": {
			"name": "nome"
		}
	};

	it("I18nEngine should exists", function() {
		expect(I18nEngine).not.toEqual(undefined);
	});

	it("Should translate", function(done) {
		var i18n = new I18nEngine(_$injector);

		var requestMetaDataTimesCalled = 0;
		restEngine.requestMetaData = function(url) {
			requestMetaDataTimesCalled++;
			var response = {
				data: {}
			};

			switch(requestMetaDataTimesCalled) {
				case 1:
					expect(url).toBe('json/words/pt_br.json');
					response.data = words.pt_br;
					break;
				case 2:
					expect(url).toBe('json/words/es.json');
					response.data = words.es;
					break;
			}

			return Q.when(response);
		};

		spyOn(eventAggregator, 'publish');

		i18n.setLanguage('pt_br').then(function() {
			expect(eventAggregator.publish).toHaveBeenCalledWith('onChangeLanguage');
			expect(i18n.i18n('name', 'pt_br')).toEqual("nome");
		}).then(function() {
			i18n.setLanguage('es').then(function() {
				expect(eventAggregator.publish).toHaveBeenCalledWith('onChangeLanguage');
				expect(i18n.i18n('name', 'es')).toEqual("nombre");
				done();
			});
		});
	});

	it("Should translate with variables", function(done) {
		var i18n = new I18nEngine(_$injector);

		var requestMetaDataTimesCalled = 0;
		restEngine.requestMetaData = function(url) {
			requestMetaDataTimesCalled++;
			var response = {
				data: {
					words: {}
				}
			};

			switch(requestMetaDataTimesCalled) {
				case 1:
					expect(url).toBe('json/words/pt_br.json');
					response.data = words.pt_br;
					break;
				case 2:
					expect(url).toBe('json/words/en_us.json');
					response.data = words.en_us;
					break;
				case 3:
					expect(url).toBe('json/words/es.json');
					response.data = words.es;
					break;
			}

			return Q.when(response);
		};

		spyOn(eventAggregator, 'publish');

		i18n.setLanguage('pt_br').then(function() {
			expect(eventAggregator.publish).toHaveBeenCalledWith('onChangeLanguage');
			expect(i18n.i18n('key', { myVar: "variables" })).toEqual("variables are important");
		}).then(function() {
			return i18n.setLanguage('en_us');
		}).then(function() {
			expect(eventAggregator.publish).toHaveBeenCalledWith('onChangeLanguage');
			expect(i18n.i18n('key', { myVar: 5 })).toEqual("replacing with 5");
		}).then(function() {
			return i18n.setLanguage('es');
		}).then(function() {
			expect(eventAggregator.publish).toHaveBeenCalledWith('onChangeLanguage');
			expect(i18n.i18n('key', { myVar: "variables" })).toEqual("las variables son importantes");
			expect(i18n.i18n('key')).toEqual("las __myVar__ son importantes");
			done();
		});
	});

});