var BedsService = function(BedsRepository, ZHPromise) {

    this.getVacantBeds = function() {
        return BedsRepository.findFromVacantBeds();
    };
    this.getOccupiedBeds = function() {
        return BedsRepository.findFromOccupiedBeds();
    };

    this.getAllBeds = function() {
        var defer = ZHPromise.defer();
        ZHPromise.all([
            BedsRepository.findFromVacantBeds(),
            BedsRepository.findFromOccupiedBeds()
        ]).then(
            function success(values) {
                var vacantBeds = values[0] || [];
                var occupiedBeds = values[1] || [];
                var allBeds = vacantBeds.concat(occupiedBeds);
                defer.resolve(allBeds);
            },
            function error(e) {
                defer.reject(e);
            }
        );
        return defer.promise;
    };

    this.getBed = function(bed) {
        var defer = ZHPromise.defer();
        ZHPromise.all([
            BedsRepository.findOneFromVacantBeds(bed),
            BedsRepository.findOneFromOccupiedBeds(bed)
        ]).then(
            function success(values) {
                var vacantBed = values[0] || [];
                var occupiedBed = values[1] || [];
                if (vacantBed && vacantBed.bed === bed) {
                    defer.resolve(vacantBed);
                } else if (occupiedBed && occupiedBed.bed === bed) {
                    defer.resolve(occupiedBed);
                } else {
                    defer.resolve([]);
                }
            },
            function error(e) {
                defer.reject(e);
            }
        );
        return defer.promise;
    };

    this.localSwapBeds = function(vacantBedToOccupy, occupiedBedToVacate) {
        var vacantBed = Object.assign({}, vacantBedToOccupy);
        var occupiedBed = Object.assign({}, occupiedBedToVacate);

        return ZHPromise.all([
            BedsRepository.removeFromVacantBeds(vacantBed).then(function() {
                BedsRepository.saveInVacantBeds(occupiedBed);
            }),
            BedsRepository.removeFromOccupiedBeds(occupiedBed).then(function() {
                BedsRepository.saveInOccupiedBeds(vacantBed);
            })
        ]);
    };

    this.changeVacantBedToOccupied = function(vacantBedToOccupy) {
        var vacantBed = Object.assign({}, vacantBedToOccupy);

        return ZHPromise.all([
            BedsRepository.removeFromVacantBeds(vacantBed).then(function() {
                BedsRepository.saveInOccupiedBeds(vacantBed);
            })
        ]);
    };

    this.changeOccupiedBedToVacate = function(occupiedBedToVacate) {
        var occupiedBed = Object.assign({}, occupiedBedToVacate);

        return ZHPromise.all([
            BedsRepository.removeFromOccupiedBeds(occupiedBed).then(function() {
                BedsRepository.saveInVacantBeds(occupiedBed);
            })
        ]);
    };
};
Configuration(function(ContextRegister) {
    ContextRegister.register('BedsService', BedsService);
});
